#!/usr/bin/perl

use strict;

my $fdr_thresh = $ARGV[0] || 0.001;
my $e_thresh   = $ARGV[1] || 100;
my $ga = 0;

if ($#ARGV < 1) {
   die "$0 fwd_file fp_file [fdr_thresh] [e_thresh]  (fdr default 0.001, e default 100)\n";
} 

my ($fwd_file, $fp_file, $fdr_thresh, $e_thresh) = @ARGV;  

$fdr_thresh ||=  0.001; 
$e_thresh   ||=  100;

my %score2evals;

my $min_score = 10000000;
my $line;
my $score;
my %fwd_scores;
my %fwd_lengths;
open F, "<$fwd_file";
my $i=0;
while ($line = <F>) {
   my @vals = split (/\s+/, $line);	
   $fwd_scores{$i}  = $vals[6];
   $fwd_lengths{$i} = $vals[2]-$vals[1]+1;
   $score2evals{$vals[6]} = $vals[7];  # I'm just looking for a translation here, don't need to keep track of which line gave the e-val
   if ($vals[6] < $min_score) {
      $min_score = $vals[6];
   }
   $i++;
}


my %fp_scores;
my %fp_lengths;
open F, "<$fp_file";
$i=0;
while ($line = <F>) {
   my @vals = split (/\s+/, $line);	
   $fp_scores{$i}  =  $vals[6];
   $fp_lengths{$i} =  $vals[2]-$vals[1]+1;   
   $score2evals{$vals[6]} = $vals[7];  # I'm just looking for a translation here, don't need to keep track of which line gave the e-val
   $i++;
}


my @fwd_keys = sort {$fwd_scores{$b} <=> $fwd_scores{$a}} keys %fwd_scores;
my @fp_keys = sort {$fp_scores{$b} <=> $fp_scores{$a}} keys %fp_scores;

my $i = 0;
my $j = 0;

#print "($#fp_scores , $#fwd_scores)\n";

if ($#fp_keys < 0 && $#fwd_keys < 0) {
   $ga = "25 *"; #there's no hits to base it on
} elsif ($#fp_keys < 0) {
   while ($i <= $#fwd_keys && $score2evals{$fwd_scores{$fwd_keys[$i]}} <= $e_thresh) { 
      $i++;
   }
   $ga = "$fwd_scores{$fwd_keys[$i-1]} ** $i";
} elsif ($#fwd_keys < 0) {
   $ga = "" . ($fp_scores{$fp_keys[0]} + 0.05) . " ***";
} else {
   
   my $fwd_base_cnt = 0;
   my $fp_base_cnt  = 0;   
   $ga = $fp_scores{$fp_keys[0]} + 0.05; # in case the highest scoring hit is an FP
   
   while ($i <= $#fwd_keys && $j <= $#fp_keys && $score2evals{$fwd_scores{$fwd_keys[$i]}} <= $e_thresh) {
      #print " $score2evals{$fwd_scores{$fwd_keys[$i]}} < $e_thresh,    $fwd_scores{$fwd_keys[$i]} > $fp_scores{$fp_keys[$j]} \n";
      #print " $i,$j |  $fwd_keys[$i],$fp_keys[$j] |  $fwd_scores{$fwd_keys[$i]} v $fp_scores{$fp_keys[$j]}\n";
      if ( $fwd_scores{$fwd_keys[$i]} > $fp_scores{$fp_keys[$j]} ) {
         $fwd_base_cnt += $fwd_lengths{$fwd_keys[$i]};
         $i++;
      } else {
         $fp_base_cnt += $fp_lengths{$fp_keys[$j]};
         $j++;
      } 
      #print "  $fp_base_cnt/$fwd_base_cnt = " . ($fp_base_cnt/$fwd_base_cnt) . "\n";
      if ($j <= $#fp_keys && $j <10 && ( $fwd_base_cnt > 0 && $fp_base_cnt/$fwd_base_cnt < $fdr_thresh) ) {
         $ga = $fp_scores{$fp_keys[$j]} + 0.05; 
         #print "  new ga: $ga\n";
      } 
   }


   if ($i <= $#fwd_keys) { 
      if ( $score2evals{$fwd_scores{$fwd_keys[$i]}} > $e_thresh ) {
         #e-value threshold overrides other GA method
         #print "override 1\n";
         $ga = $fwd_scores{$fwd_keys[$i]}+0.05 if $fwd_scores{$fwd_keys[$i]}+0.05 > $ga;
      } elsif ($fwd_base_cnt == 0 || ($j<10 && $fp_base_cnt/$fwd_base_cnt < $fdr_thresh)) { 
         #exhausted fps before exhausting tps, and the ratio of fps to tps is still acceptable
         # this happens in the case of no fps
         #print "override 2\n";
         $ga = $fwd_scores{$fwd_keys[-1]} ;
      }
   }      
}




$fwd_file =~ s[.+/][];
$fwd_file =~ s[\..+$][];
print "$fwd_file $ga";
#print "    ($min_score)" if $min_score != $ga;
print "\n";

