#!/usr/bin/perl

$file = $ARGV[0];

my $type;
if ($file =~ /cross_match/ ){ 
   $type = "CM";
} elsif ($file =~ /blast/ ) {
   $type = "blast";
} elsif ($file =~ /hmmer/ ) {
   $type = "nhmmer";
} else {
   die "unsure what kind of file $file is\n";
}


open FH, "<$file";
   
      
while ($line = <FH>) {
   if ($type eq "nhmmer" or $type eq "blast") {
     next if $line =~ /^#/;
   } else {
	 next if $line !~ /^\s*\d+\s+\d/;  # I love parsing CM output.
   }
   
   $line =~ s/^\s+//; #deal with CM's leading spaces
   my @vals = split (/\s+/, $line);
	
   $orientation = "+";
   if ($type eq "nhmmer") {
      ($target, $t_start, $t_end, $q_start, $q_end, $score, $evalue) = @vals[0,9,10,6,7,3,4];
   } elsif ($type eq "blast") {
      ($target, $t_start, $t_end, $q_start, $q_end, $score, $evalue) = @vals[1,8,9,6,7,11,10];
   } else { #CM

      ($target, $t_start, $t_end, $score) = @vals[4,5,6,0]; 

      #CM format is non-uniform, so have to work around the occasional "C" value
      if ($vals[8] eq "C") {  
         ($q_start, $q_end) = @vals[12,11];
         $orientation = "-";          
      } else {
         ($q_start, $q_end) = @vals[9,10];
      }      
   }
   
   if ($type ne "cross_match"  &&  $t_start > $t_end) {
      my $tmp  = $t_start;
      $t_start = $t_end;
      $t_end   = $tmp;
      $orientation = "-";
   }

   $line = sprintf ("%-50s %5d %5d %4d %4d %s %6.1f", $target, $t_start, $t_end, $q_start, $q_end, $orientation, $score);
   if ($type ne "cross_match") {
      $line .= " $evalue";
   }
   
   
   $line_targets{$line} = $target;
   $line_orients{$line} = $orientation;   
   $line_starts{$line} = $t_start;   
}


foreach $key ( sort by_targetandstart keys %line_targets ) {
   print "$key\n";
}


sub by_targetandstart {
   return (   ($line_targets{$a} cmp $line_targets{$b}) 
           || ($line_orients{$a} cmp $line_orients{$b})
           || ($line_starts{$a} <=> $line_starts{$b}) );
}
